function [im2, bwx] = imSimNN(imt, net, TH);
%
% Neural network image segmentation function (evaluation part)
% Author: (c) 2007, Jan Vanek, vanekyj@kky.zcu.cz
% Presented on: Internation Conference of Technical Computing, Prague 2007.
% Using, copying and modification allowed for study and research purposes.
% Reference on my paper have to be cited if you use this function in all
%   publication
% If any questions, email me, please.
%
%image segmentation via trained neural network
%[out,bwx] = imSimNN(imt, net, TH);
%
%imt is an input image (in double type)
%net is a trained neural network witj tansig output transfer function
%TH is an threshold (default 0.5), from interval <0,1>
%
%im2 is an output image with results of neural network transformed to <0,1> interval
%bwx is a output binary image (thresholded)

if nargin < 3
    TH = 0.5;
end
q = size(imt);
if q(3) ~= net.inputs{1}.size
   disp('Input image must have the same dimesion like neural network')
   bwx = zeros(q(1), q(2));
   return
end
o = zeros(q(1)*q(2), 3); %temporary array for neural network evaluation

%copy data into tmp array
for k = 1:q(3)
    o(:, k) = reshape(imt(:,:,k), q(1)*q(2), 1);
end

%neural network evaluation
oo = sim(net, o')';

im2 = reshape(oo/2 + .5, q(1), q(2));
bwx = im2 > .5;


