function net = imTrainNN(im, bw, epochs, perceptrons);
%
% Neural network image segmentation function (training part)
% Author: (c) 2007, Jan Vanek, vanekyj@kky.zcu.cz
% Presented on: Internation Conference of Technical Computing, Prague 2007.
% Using, copying and modification allowed for study and research purposes.
% Reference on my paper have to be cited if you use this function in all
%   publication
% If any questions, email me, please.
%
%train a neural network for image segmentation
%Syntax: net = imTrainNN(im, bw, epochs, perceptrons);
%im is an input image - (usually 3 dimensional image (e.g. rgb, hsv)
%bw is an imput reference binary image, where background = 0, objects = 1 (pixels with other values are ignored) 
%epoch is a number of trainig iterations (default 100)
%perceptrons is a number of perceptron in hidden layer (default = input dimension)

if nargin < 4
   perceptrons = size(im, 3); 
end
if nargin < 3
   epochs = 100; 
end

%prepare data, normalize number of training vectors of each claasses
%from bigger class, only needed number of vectors is randomly selected
[f0x, f0y] = find(bw == 0);
[f1x, f1y] = find(bw == 1);
L0 = length(f0x);
L1 = length(f1x);
rp = randperm(max(L0, L1));
if L0 > L1
    f0x = f0x(rp(1:L1));
    f0y = f0y(rp(1:L1));
    Lmn = L1;
else
    f1x = f1x(rp(1:L0));
    f1y = f1y(rp(1:L0));
    Lmn = L0;
end
p0 = zeros(length(f0x), 3);
p1 = zeros(length(f1x), 3);
for k = 1:length(f0x)
    p0(k, :) = reshape(im(f0x(k),f0y(k),:), 1, 3);
    p1(k, :) = reshape(im(f1x(k),f1y(k),:), 1, 3);
end
tr = [p0; p1];
ref = [-ones(Lmn,1); ones(Lmn, 1)];
rp2 = randperm(2*Lmn);
tr = tr(rp2, :); %randomly mixed trainig vectors/pixels
ref = ref(rp2);

%net initiolisation
net = newff(minmax(tr'), [perceptrons, 1]);
net.trainParam.epochs = epochs;
net.trainParam.goal = 0.01;
%train net
net = train(net, tr', ref');



